@@ -52,7 +52,8 @@ extern struct termios shell_modes;
 extern wchar_t ellipsis_char;
 
 /**
-   The verbosity of fish
+   The verbosity level of fish. If a call to debug has a severity
+   level higher than \c debug_level, it will not be printed.
 */
 extern int debug_level;
 
@@ -71,30 +72,58 @@ extern wchar_t *program_name;
    This macro is used to check that an input argument is not null. It
    is a bit lika a non-fatal form of assert. Instead of exit-ing on
    failiure, the current function is ended at once. The second
-   parameter is the exit status of the current function on failiure.
+   parameter is the return value of the current function on failiure.
 */
-#define CHECK( arg, retval )					\
+#define CHECK( arg, retval )											\
 	if( !(arg) )														\
 	{																	\
 		debug( 1,														\
-			   L"function %s called with null value for argument %s. "	\
-			   L"This is a bug. "										\
-			   L"If you can reproduce it, please send a bug report to %s.", \
+			   _( L"function %s called with null value for argument %s. " \
+				  L"This is a bug. "									\
+				  L"If you can reproduce it, please send a bug report to %s." ), \
 			   __func__,												\
 			   #arg,													\
 			   PACKAGE_BUGREPORT );										\
 		return retval;													\
-	}																	\
-		
+	}
 
 /**
-   Exit program at once, leaving an error message about running out of memory
+   Exit program at once, leaving an error message about running out of memory.
 */
-#define DIE_MEM()								\
+#define DIE_MEM()														\
 	{																	\
-		fwprintf( stderr, L"fish: Out of memory on line %d of file %s, shutting down fish\n", __LINE__, __FILE__ );	\
+		fwprintf( stderr,												\
+				  L"fish: Out of memory on line %d of file %s, shutting down fish\n", \
+				  __LINE__,												\
+				  __FILE__ );											\
 		exit(1);														\
-	}																	\
+	}
+
+/**
+   Cause fish to crash. This should only be used for debugging. If
+   this function is ever called in shipped code, this is a bug.
+*/
+#define CRASH()									\
+	{											\
+		int *n = 0;								\
+		*n = 1;									\
+	}
+
+/**
+   Check if signals are blocked. If so, print an error message and
+   return from the function performing this check.
+*/
+#define CHECK_BLOCK( retval )													\
+	if( signal_is_blocked() )											\
+	{																	\
+		debug( 0,														\
+			   L"function %s called while blocking signals. "			\
+			   L"This is a bug. "										\
+			   L"If you can reproduce it, please send a bug report to %s.",	\
+			   __func__,												\
+			   PACKAGE_BUGREPORT );										\
+		return retval;											\
+	}
 		
 /**
    Shorthand for wgettext call
@@ -102,10 +131,12 @@ extern wchar_t *program_name;
 #define _(wstr) wgettext(wstr)
 
 /**
-   Noop, used to tell xgettext that a string should be translated, even though it is not directly sent to wgettext.
+   Noop, used to tell xgettext that a string should be translated,
+   even though it is not directly sent to wgettext. 
 */
 #define N_(wstr) wstr
 
+
 /**
    Take an array_list_t containing wide strings and converts them to a
    single null-terminated wchar_t **. The array is allocated using
@@ -193,7 +224,7 @@ wchar_t *wcsdupcat( const wchar_t *a, const wchar_t *b );
    Returns a newly allocated concatenation of the specified wide
    character strings. The last argument must be a null pointer.
 */
-wchar_t *wcsdupcat2( const wchar_t *a, ... );
+__sentinel wchar_t *wcsdupcat2( const wchar_t *a, ... );
 
 /**
    Test if the given string is a valid variable name
@@ -206,7 +237,16 @@ wchar_t *wcsdupcat2( const wchar_t *a, ... );
    \return null if this is a valid name, and a pointer to the first invalid character otherwise
 */
 
-wchar_t *wcsvarname( wchar_t *str );
+wchar_t *wcsvarname( const wchar_t *str );
+
+
+/**
+   Test if the given string is a valid function name. 
+
+   \return null if this is a valid name, and a pointer to the first invalid character otherwise
+*/
+
+wchar_t *wcsfuncname( const wchar_t *str );
 
 /**
    Test if the given string is valid in a variable name 
@@ -256,7 +296,7 @@ const wchar_t *wsetlocale( int category, const wchar_t *locale );
 
    \return zero is needle is not found, of if needle is null, non-zero otherwise
 */
-int contains_str( const wchar_t *needle, ... );
+__sentinel int contains_str( const wchar_t *needle, ... );
 
 /**
    Call read while blocking the SIGCHLD signal. Should only be called
@@ -358,5 +398,12 @@ void write_screen( const wchar_t *msg, string_buffer_t *buff );
 void tokenize_variable_array( const wchar_t *val, array_list_t *out );
 
 
+/**
+   Make sure the specified direcotry exists. If no, try to create it.
+
+   \return 0 if the directory exists, -1 otherwise.
+*/
+int create_directory( wchar_t *d );
+
 #endif
 
