@@ -655,8 +655,8 @@ static int expand_pid( wchar_t *in,
 			free( in );
 		else
 		{
-			*in = L'%';
-			al_push( out, in );
+			free(in);
+			return 0;
 		}
 	}
 	else
@@ -1265,7 +1265,12 @@ static int expand_cmdsubst( wchar_t *in, array_list_t *out )
 	wcslcpy( subcmd, paran_begin+1, paran_end-paran_begin );
 	subcmd[ paran_end-paran_begin-1]=0;
 
-	exec_subshell( subcmd, sub_res);
+	if( exec_subshell( subcmd, sub_res) == -1 )
+	{
+		halloc_free( context );
+		error( CMDSUBST_ERROR, -1, L"Unknown error while evaulating command substitution" );
+		return 0;
+	}
 
 	tail_begin = paran_end + 1;
 	if( *tail_begin == L'[' )
@@ -1307,22 +1312,21 @@ static int expand_cmdsubst( wchar_t *in, array_list_t *out )
 			}
 			al_foreach( sub_res, &free );
 			sub_res = sub_res2;			
-		}		
-		
+		}
 	}
 
 	tail_expand = al_halloc( context );
 
 	/*
 	  Recursively call ourselves to expand any remaining command
-	  substitutions. The result of this recusrive call usiung the tail
+	  substitutions. The result of this recursive call usiung the tail
 	  of the string is inserted into the tail_expand array list
 	*/
 	expand_cmdsubst( wcsdup(tail_begin), tail_expand );
 
 	/*
 	  Combine the result of the current command substitution with the
-	  result of the recusrive tail expansion
+	  result of the recursive tail expansion
 	*/
     for( i=0; i<al_get_count( sub_res ); i++ )
     {
