@@ -5,6 +5,8 @@
 
 	Type ^C to exit the program.
 */
+#include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -13,6 +15,8 @@
 #include <locale.h>
 #include <termcap.h>
 
+#include "fallback.h"
+
 #include "input_common.h"
 
 int writestr( char *str )
