@@ -135,6 +135,11 @@ void output_set_writer( int (*writer)(char) )
 	out = writer;
 }
 
+int (*output_get_writer())(char)
+{
+	return out;
+}
+
 
 void set_color( int c, int c2 )
 {
@@ -336,34 +341,6 @@ void set_color( int c, int c2 )
 	
 }
 
-
-/**
-   perm_left_cursor and parm_right_cursor don't seem to be properly
-   emulated by many terminal emulators, so we only use plain
-   curor_left, curor_right...
-*/
-void move_cursor( int steps )
-{
-	int i;
-	
-	if( !steps )
-		return;
-	
-	if( steps < 0 ){
-		for( i=0; i>steps; i--)
-		{
-			writembs(cursor_left);
-		}
-	}
-	else
-	{
-		for( i=0; i<steps; i++)
-		{
-			writembs(cursor_right);
-		}
-	}
-}
-
 /**
    Default output method, simply calls write() on stdout
 */
@@ -553,25 +530,6 @@ int write_escaped_str( const wchar_t *str, int max_len )
 }
 
 
-int writespace( int c )
-{
-	if( repeat_char && strlen(repeat_char) )
-	{
-		writembs( tparm( repeat_char, ' ', c ) );
-	}
-	else
-	{
-		int i;
-		
-		for( i=0; i<c; i++ )
-		{
-			out( ' ' );
-		}
-	}
-	return 0;
-}
-
-
 int output_color_code( const wchar_t *val )
 {
 	int j, i, color=FISH_COLOR_NORMAL;
