@@ -189,9 +189,8 @@ void proc_destroy()
 
 void proc_set_last_status( int s )
 {
-
 	last_status = s;
-	//	fwprintf( stderr, L"Set last status to %d\n", s );
+//	debug( 0, L"Set last status to %d\n", s );
 }
 
 int proc_get_last_status()
@@ -211,8 +210,10 @@ job_t *job_create()
 	res->job_id = free_id;
 	first_job = res;
 
-	res->job_control = (job_control_mode==JOB_CONTROL_ALL) || 
-		((job_control_mode == JOB_CONTROL_INTERACTIVE) && (is_interactive));
+	job_set_flag( res, 
+				  JOB_CONTROL, 
+				  (job_control_mode==JOB_CONTROL_ALL) || 
+				  ((job_control_mode == JOB_CONTROL_INTERACTIVE) && (is_interactive)) );
 
 //	if( res->job_id > 2 )
 //		fwprintf( stderr, L"Create job %d\n", res->job_id );	
@@ -287,6 +288,20 @@ int job_is_completed( const job_t *j )
 	
 }
 
+void job_set_flag( job_t *j, int flag, int set )
+{
+	if( set )
+		j->flags |= flag;
+	else
+		j->flags = j->flags & (0xffffffff ^ flag);
+}
+
+int job_get_flag( job_t *j, int flag )
+{
+	return j->flags&flag?1:0;
+}
+
+
 /**
    Store the status of the process pid that was returned by waitpid.
    Return 0 if all went well, nonzero otherwise.  
@@ -296,6 +311,8 @@ static void mark_process_status( job_t *j,
 								 int status )
 {
 	p->status = status;
+//	debug( 0, L"Process %ls %ls", p->argv[0], WIFSTOPPED (status)?L"stopped":(WIFEXITED( status )?L"exited":(WIFSIGNALED( status )?L"signaled to exit":L"BLARGH")) );
+	
 	if (WIFSTOPPED (status))
 	{
 		p->stopped = 1;
@@ -303,6 +320,7 @@ static void mark_process_status( job_t *j,
 	else
 	{
 		p->completed = 1;
+
 		
 		if (( !WIFEXITED( status ) ) &&
 			(! WIFSIGNALED( status )) )
@@ -521,7 +539,7 @@ int job_reap( int interactive )
 		  sent to the console, do not consider reaping jobs that need
 		  status messages
 		*/
-		if( (!j->skip_notification) && (!interactive) && (!j->fg))
+		if( (!job_get_flag( j, JOB_SKIP_NOTIFICATION ) ) && (!interactive) && (!job_get_flag( j, JOB_FOREGROUND )))
 		{
 			continue;
 		}
@@ -549,8 +567,8 @@ int job_reap( int interactive )
 				{	
 					int proc_is_job = ((p==j->first_process) && (p->next == 0));
 					if( proc_is_job )
-						j->notified = 1;
-					if( !j->skip_notification )
+						job_set_flag( j, JOB_NOTIFIED, 1 );
+					if( !job_get_flag( j, JOB_SKIP_NOTIFICATION ) )
 					{
 						if( proc_is_job )
 							fwprintf( stdout,
@@ -589,30 +607,27 @@ int job_reap( int interactive )
 		*/
 		if( job_is_completed( j ) ) 
 		{
-			if( !j->fg && !j->notified )
+			if( !job_get_flag( j, JOB_FOREGROUND) && !job_get_flag( j, JOB_NOTIFIED ) && !job_get_flag( j, JOB_SKIP_NOTIFICATION ) )
 			{
-				if( !j->skip_notification )
-				{
-					format_job_info( j, _( L"ended" ) );
-					found=1;
-				}
+				format_job_info( j, _( L"ended" ) );
+				found=1;
 			}
 			proc_fire_event( L"JOB_EXIT", EVENT_EXIT, -j->pgid, 0 );			
 			proc_fire_event( L"JOB_EXIT", EVENT_JOB_ID, j->job_id, 0 );			
 
 			job_free(j);
 		}		
-		else if( job_is_stopped( j ) && !j->notified ) 
+		else if( job_is_stopped( j ) && !job_get_flag( j, JOB_NOTIFIED ) ) 
 		{
 			/* 
 			   Notify the user about newly stopped jobs. 
 			*/
-			if( !j->skip_notification )
+			if( !job_get_flag( j, JOB_SKIP_NOTIFICATION ) )
 			{
 				format_job_info( j, _( L"stopped" ) );
 				found=1;
 			}			
-			j->notified = 1;
+			job_set_flag( j, JOB_NOTIFIED, 1 );
 		}
 	}
 
@@ -835,6 +850,70 @@ static void read_try( job_t *j )
 }
 
 
+static int terminal_give_to_job( job_t *j, int cont )
+{
+	
+	if( tcsetpgrp (0, j->pgid) )
+	{
+		debug( 1, 
+			   _( L"Could not send job %d ('%ls') to foreground" ), 
+			   j->job_id, 
+			   j->command );
+		wperror( L"tcsetpgrp" );
+		return 0;
+	}
+	
+	if( cont )
+	{  
+		if( tcsetattr (0, TCSADRAIN, &j->tmodes))
+		{
+			debug( 1,
+				   _( L"Could not send job %d ('%ls') to foreground" ),
+				   j->job_id,
+				   j->command );
+			wperror( L"tcsetattr" );
+			return 0;
+		}
+	}
+	return 1;
+}
+
+/**
+   REturns contol of the terminal 
+*/
+static int terminal_return_from_job( job_t *j)
+{
+		
+	if( tcsetpgrp (0, getpid()) )
+	{
+		debug( 1, _( L"Could not return shell to foreground" ) );
+		wperror( L"tcsetpgrp" );
+		return 0;
+	}
+	
+	/* 
+	   Save jobs terminal modes.  
+	*/
+	if( tcgetattr (0, &j->tmodes) )
+	{
+		debug( 1, _( L"Could not return shell to foreground" ) );
+		wperror( L"tcgetattr" );
+		return 0;
+	}
+	
+	/* 
+	   Restore the shell's terminal modes.  
+	*/
+	if( tcsetattr (0, TCSADRAIN, &shell_modes))
+	{
+		debug( 1, _( L"Could not return shell to foreground" ) );
+		wperror( L"tcsetattr" );
+		return 0;
+	}
+
+	return 1;
+}
+
 void job_continue (job_t *j, int cont)
 {
 	/*
@@ -843,46 +922,36 @@ void job_continue (job_t *j, int cont)
 	job_remove( j );
 	j->next = first_job;
 	first_job = j;
-	j->notified = 0;
+	job_set_flag( j, JOB_NOTIFIED, 0 );
 
+	CHECK_BLOCK();
+	
 	debug( 4,
-		   L"Continue job %d (%ls), %ls, %ls",
+		   L"Continue job %d, gid %d (%ls), %ls, %ls",
 		   j->job_id, 
+		   j->pgid,
 		   j->command, 
 		   job_is_completed( j )?L"COMPLETED":L"UNCOMPLETED", 
 		   is_interactive?L"INTERACTIVE":L"NON-INTERACTIVE" );
 	
 	if( !job_is_completed( j ) )
 	{
-		if( j->terminal && j->fg )
+		if( job_get_flag( j, JOB_TERMINAL ) && job_get_flag( j, JOB_FOREGROUND ) )
 		{							
 			/* Put the job into the foreground.  */
+			int ok;
+			
 			signal_block();
-			if( tcsetpgrp (0, j->pgid) )
-			{
-				debug( 1, 
-					   _( L"Could not send job %d ('%ls') to foreground" ), 
-					   j->job_id, 
-					   j->command );
-				wperror( L"tcsetpgrp" );
-				return;
-			}
+			
+			ok = terminal_give_to_job( j, cont );
 			
-			if( cont )
-			{  
-				if( tcsetattr (0, TCSADRAIN, &j->tmodes))
-				{
-					debug( 1,
-						   _( L"Could not send job %d ('%ls') to foreground" ),
-						   j->job_id,
-						   j->command );
-					wperror( L"tcsetattr" );
-					return;
-				}										
-			}
 			signal_unblock();		
-		}
 
+			if( !ok )
+				return;
+			
+		}
+		
 		/* 
 		   Send the job a continue signal, if necessary.  
 		*/
@@ -893,7 +962,7 @@ void job_continue (job_t *j, int cont)
 			for( p=j->first_process; p; p=p->next )
 				p->stopped=0;
 
-			if( j->job_control )
+			if( job_get_flag( j, JOB_CONTROL ) )
 			{
 				if( killpg( j->pgid, SIGCONT ) )
 				{
@@ -914,11 +983,10 @@ void job_continue (job_t *j, int cont)
 			}
 		}
 	
-		if( j->fg )
+		if( job_get_flag( j, JOB_FOREGROUND ) )
 		{
 			int quit = 0;
 		
-//			debug( 1, L"wait loop" );	
 			/* 
 			   Wait for job to report. Looks a bit ugly because it has to
 			   handle the possibility that a signal is dispatched while
@@ -955,10 +1023,27 @@ void job_continue (job_t *j, int cont)
 							  short-lived jobs.
 							*/
 							int status;						
-//							debug( 1, L"waitpid" );	
 							pid_t pid = waitpid(-1, &status, WUNTRACED );
 							if( pid > 0 )
+							{
 								handle_child_status( pid, status );
+							}
+							else
+							{
+								/*
+								  This probably means we got a
+								  signal. A signal might mean that the
+								  terminal emulator sent us a hup
+								  signal to tell is to close. If so,
+								  we should exit.
+								*/
+								if( reader_exit_forced() )
+								{
+									quit = 1;
+								}
+								
+							}
+							
 							break;
 						}
 								
@@ -968,7 +1053,7 @@ void job_continue (job_t *j, int cont)
 		}	
 	}
 	
-	if( j->fg )
+	if( job_get_flag( j, JOB_FOREGROUND ) )
 	{
 		
 		if( job_is_completed( j ))
@@ -986,43 +1071,26 @@ void job_continue (job_t *j, int cont)
 				if( p->pid )
 				{
 					debug( 3, L"Set status of %ls to %d", j->command, WEXITSTATUS(p->status) );
-					proc_set_last_status( j->negate?(WEXITSTATUS(p->status)?0:1):WEXITSTATUS(p->status) );
+					proc_set_last_status( job_get_flag( j, JOB_NEGATE )?(WEXITSTATUS(p->status)?0:1):WEXITSTATUS(p->status) );
 				}
 			}			
 		}
 		/* 
 		   Put the shell back in the foreground.  
 		*/
-		if( j->terminal && j->fg )
+		if( job_get_flag( j, JOB_TERMINAL ) && job_get_flag( j, JOB_FOREGROUND ) )
 		{
+			int ok;
+			
 			signal_block();
-			if( tcsetpgrp (0, getpid()) )
-			{
-				debug( 1, _( L"Could not return shell to foreground" ) );
-				wperror( L"tcsetpgrp" );
-				return;
-			}
+
+			ok = terminal_return_from_job( j );
 			
-			/* 
-			   Save jobs terminal modes.  
-			*/
-			if( tcgetattr (0, &j->tmodes) )
-			{
-				debug( 1, _( L"Could not return shell to foreground" ) );
-				wperror( L"tcgetattr" );
-				return;
-			}
-			
-			/* 
-			   Restore the shell's terminal modes.  
-			*/
-			if( tcsetattr (0, TCSADRAIN, &shell_modes))
-			{
-				debug( 1, _( L"Could not return shell to foreground" ) );
-				wperror( L"tcsetattr" );
-				return;
-			}
 			signal_unblock();
+			
+			if( !ok )
+				return;
+			
 		}
 	}
 	
@@ -1037,7 +1105,7 @@ void proc_sanity_check()
 	{
 		process_t *p;
 
-		if( !j->constructed )
+		if( !job_get_flag( j, JOB_CONSTRUCTED ) )
 			continue;
 		
 		
@@ -1054,7 +1122,7 @@ void proc_sanity_check()
 		/*
 		  More than one foreground job?
 		*/
-		if( j->fg && !(job_is_stopped(j) || job_is_completed(j) ) )
+		if( job_get_flag( j, JOB_FOREGROUND ) && !(job_is_stopped(j) || job_is_completed(j) ) )
 		{
 			if( fg_job != 0 )
 			{
