@@ -1,6 +1,8 @@
 /** \file sanity.c
 	Functions for performing sanity checks on the program state
 */
+#include "config.h"
+
 #include <stdlib.h>
 #include <wchar.h>
 #include <stdio.h>
@@ -13,8 +15,9 @@
 #include <dirent.h>
 
 
-#include "config.h"
+#include "fallback.h"
 #include "util.h"
+
 #include "common.h"
 #include "sanity.h"
 #include "proc.h"
@@ -22,7 +25,7 @@
 #include "reader.h"
 #include "kill.h"
 #include "wutil.h"
-#include "translate.h"
+
 
 /**
    Status from earlier sanity checks
@@ -57,7 +60,7 @@ void validate_pointer( const void *ptr, const wchar_t *err, int null_ok )
 	   Test if the pointer data crosses a segment boundary. 
 	*/
 	
-	if( (0x00000003 & (int)ptr) != 0 )
+	if( (0x00000003l & (long)ptr) != 0 )
 	{
 		debug( 0, _(L"The pointer '%ls' is invalid"), err );
 		sanity_lose();		
