@@ -1,5 +1,6 @@
 #include "config.h"
 
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -7,6 +8,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <locale.h>
 
 #if HAVE_NCURSES_H
 #include <ncurses.h>
@@ -14,18 +16,28 @@
 #include <curses.h>
 #endif
 
-
 #if HAVE_TERMIO_H
 #include <termio.h>
 #endif
 
+#if HAVE_TERM_H
 #include <term.h>
+#elif HAVE_NCURSES_TERM_H
+#include <ncurses/term.h>
+#endif
+
 #include <errno.h>
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#if HAVE_LIBINTL_H
+#include <libintl.h>
+#endif
+
+#include "fallback.h"
+
 /*
   Small utility for setting the color.
   Usage: set_color COLOR
@@ -34,6 +46,18 @@
 
 #define COLORS (sizeof(col)/sizeof(char *))
 
+/**
+   Program name
+*/
+#define SET_COLOR "set_color"
+
+/**
+   Getopt short switches for set_color
+*/
+#define GETOPT_STRING "b:hvocu"
+
+#define _(string) gettext(string)
+
 char *col[]=
 {
 	"black",
@@ -105,16 +129,30 @@ void print_colors()
 	}	
 }
 
+static void check_locale_init()
+{
+	static int is_init = 0;
+	if( is_init )
+		return;
+	
+	is_init = 1;
+	setlocale( LC_ALL, "" );
+	bindtextdomain( PACKAGE_NAME, LOCALEDIR );
+	textdomain( PACKAGE_NAME );
+}
+
+
 int main( int argc, char **argv )
 {
 	char *bgcolor=0;
 	char *fgcolor=0;
 	int fg, bg;
 	int bold=0;
-		
+	int underline=0;
+	char *bg_seq, *fg_seq;
+			
 	while( 1 )
 	{
-#ifdef HAVE_GETOPT_LONG
 		static struct option
 			long_options[] =
 			{
@@ -130,6 +168,10 @@ int main( int argc, char **argv )
 					"bold", no_argument, 0, 'o' 
 				}
 				,
+				{
+					"underline", no_argument, 0, 'u' 
+				}
+				,
 				{
 					"version", no_argument, 0, 'v' 
 				}
@@ -148,14 +190,10 @@ int main( int argc, char **argv )
 		
 		int opt = getopt_long( argc,
 							   argv, 
-							   "b:hvoc", 
+							   GETOPT_STRING, 
 							   long_options, 
 							   &opt_index );
-#else
-		int opt = getopt( argc,
-						  argv, 
-						  "b:hvoc" );
-#endif
+
 		if( opt == -1 )
 			break;
 			
@@ -175,8 +213,13 @@ int main( int argc, char **argv )
 				bold=1;
 				break;
 				
+			case 'u':
+				underline=1;
+				break;
+				
 			case 'v':
-				fprintf( stderr, "set_color, version %s\n", PACKAGE_VERSION );
+				check_locale_init();
+				fprintf( stderr, _("%s, version %s\n"), SET_COLOR, PACKAGE_VERSION );
 				exit( 0 );								
 
 			case 'c':
@@ -202,13 +245,15 @@ int main( int argc, char **argv )
 			break;
 			
 		default:
-			printf( "set_color: Too many arguments\n" );
+			check_locale_init();
+			printf( _("%s: Too many arguments\n"), SET_COLOR );
 			return 1;
 	}
 
-	if( !fgcolor && !bgcolor && !bold )
+	if( !fgcolor && !bgcolor && !bold && !underline )
 	{
-		fprintf( stderr, "set_color: Expected an argument\n" );
+		check_locale_init();
+		fprintf( stderr, _("%s: Expected an argument\n"), SET_COLOR );
 		print_help();		
 		return 1;
 	}
@@ -216,29 +261,43 @@ int main( int argc, char **argv )
 	fg = translate_color(fgcolor);
 	if( fgcolor && (fg==-1))
 	{
-		fprintf( stderr, "set_color: Unknown color %s\n", fgcolor );
+		check_locale_init();
+		fprintf( stderr, _("%s: Unknown color '%s'\n"), SET_COLOR, fgcolor );
 		return 1;		
 	}
 
 	bg = translate_color(bgcolor);
 	if( bgcolor && (bg==-1))
 	{
-		fprintf( stderr, "set_color: Unknown color %s\n", bgcolor );
+		check_locale_init();
+		fprintf( stderr, _("%s: Unknown color '%s'\n"), SET_COLOR, bgcolor );
 		return 1;		
 	}
 
 	setupterm( 0, STDOUT_FILENO, 0);
 
+	fg_seq = set_a_foreground?set_a_foreground:set_foreground;
+	bg_seq = set_a_background?set_a_background:set_background;
+	
+
 	if( bold )
 	{
-		putp( enter_bold_mode );
+		if( enter_bold_mode )
+			putp( enter_bold_mode );
+	}
+
+	if( underline )
+	{
+		if( enter_underline_mode )
+			putp( enter_underline_mode );
 	}
 
 	if( bgcolor )
 	{
 		if( bg == 8 )
 		{
-			putp( tparm( set_a_background, 0) );
+			if( bg_seq )
+				putp( tparm( bg_seq, 0) );
 			putp( tparm(exit_attribute_mode) );		
 		}	
 	}
@@ -247,12 +306,14 @@ int main( int argc, char **argv )
 	{
 		if( fg == 8 )
 		{
-			putp( tparm( set_a_foreground, 0) );
+			if( fg_seq )
+				putp( tparm( fg_seq, 0) );
 			putp( tparm(exit_attribute_mode) );		
 		}	
 		else
 		{
-			putp( tparm( set_a_foreground, fg) );
+			if( fg_seq )
+				putp( tparm( fg_seq, fg) );
 		}
 	}
 	
@@ -260,8 +321,14 @@ int main( int argc, char **argv )
 	{
 		if( bg != 8 )
 		{
-			putp( tparm( set_a_background, bg) );
+			if( bg_seq )
+				putp( tparm( bg_seq, bg) );
 		}
 	}
-	del_curterm( cur_term );
+
+	if( del_curterm( cur_term ) == ERR )
+	{
+		fprintf( stderr, _("Error while closing terminfo") );		
+	}
+
 }
